# bash range expand library 0.9.2
# (C) Copyright  Heinz Deinhart 2015-2018
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# This is a bash library, it has to be sourced not called.
#
# Written for bash < 4.4 and thus not using associative array references.
# In future version this will be added.
# Currently that means that macros will have to reside in a well known name
# (BRE_RANGE_MACROS).

[ "$0" != "$BASH_SOURCE" ] || {
	echo "$0: this script must be sourced, not called" >&2
	exit 1
}

declare -A BRE_RANGE_MACROS

#@@ range expand

#@ bre_expand_ranges <prefix> <suffix> <reserved> <input-string> [..]
#  <prefix> and <suffix> are strings without spaces
#  <reserved> is reserved for the name of the macro associative array
#             this will be implemented in a future version
#  <input-string> can be the following:
#    a range (<number>-<number>): will be expanded to each member of range
#                                 prefix and suffix will be added to each
#    a number: prefix and suffix will be added
#    a macro: a string which is key in the associative array BRE_RANGE_MACROS
#             its value will be expanded in recursive fashion
#    any other string (must not contain spaces): will be copied unmodified
# errors: 2
function bre_expand_ranges() {
    (( $# > 2 )) || {
	bb_ierr "bre_expand_ranges: called with $ instead of 3 or more args"
	return 2
    }

    local prefix="$1"
    local suffix="$2"
    # local reserved="$3"
    shift 3

    local result=""
    while (( $# > 0 ))
    do
	result="$result $(_bre_parse_range "$prefix" "$suffix" "" "$1")"
        shift
    done

    echo "${result# }"
}

### internal functions below ###

# _bre_parse_range <prefix> <suffix> <reserved> <range>
function _bre_parse_range() {
    (( $# == 4 )) || {
	bb_ierr "_bre_parse_range called with $# instead of 4 args"
	return 2
    }

    local prefix="$1"
    local suffix="$2"
    # local reserved="$3"
    local range="$4"

    # check if macros are defined
    if [[ "${BRE_RANGE_MACROS[@]-unset}" != unset ]]
    then
	# check if range is a macro
	if [ "${BRE_RANGE_MACROS[$range]+isset}" == isset ]
	then
	    bre_expand_ranges "$prefix" "$suffix" "" \
		${BRE_RANGE_MACROS[$range]}
            return 0
	fi
    fi

    # expand range
    case "$range" in
        *-*-*) # that is not a range
            echo "$range"
            ;;
        *[!0-9-]*) # that is not a range, too
            echo "$range"
            ;;
        *-*) # real range
	    local result=""
            for n in $(seq "${range%-*}" "${range#*-}")
            do
                result="$result ${prefix-}$n${suffix-}"
            done
	    echo "${result# }"
            ;;
        *) # just a number
            echo "${prefix-}$range${suffix-}"
    esac

    return 0
}
